#include "MPC5646C.h"
#include "fec.h"
#include "emios.h"
#include "ethernetframe.h"

// Declare external FEC descriptors and buffers
extern BufferDescriptor txDesc[2];
extern BufferDescriptor rxDesc[2];

extern uint8_t mac[6];
extern uint8_t broadcast[6];

extern uint8_t txBuf[1536];

// Global frame - points into the base address of the big txbuf
MyEthernetFrame *frame;

// Disable the watchdog
void disableWatchdog() 
{
    SWT.SR.R = 0xC520;          /* Clear Soft lock bit in CR by writing     */
    SWT.SR.R = 0xD928;          /*  0xC520 followed by 0xD928 to WSC field  */
    SWT.CR.B.WEN = 0x0;         /* Clear Watchdog Enable Bit -> 0xC000010A  */
}

// Initialise 64Mhz on the PLL and setup clocks/peripherals
void initModes() 
{
    ME.MER.R = 0x000025FF;          // Enable all modes		
    ME.RUNPC[0].R = 0x000000FE;     // Enable all peripherals in all modes
    
   	/* Enable system clock for all peripherals, Clock divider of 1           */
    CGM.SC_DC0.R = 0x81;
    CGM.SC_DC1.R = 0x82;
    CGM.SC_DC2.R = 0x8F;

	// Do NOT divide the FEC clock (operation < 100Mhz)
	CGM.FEC_DCR.R = 0x0;

	// Configure the PLL
   	CGM.AC0_SC.B.SELCTL = 0x1; 		// select PLL to be driven by FXOSC (40Mhz)

	// Configure run mode
	ME.RUN[0].R = 0x001F0074;		// External OSC, PLL on, SYSClk PLL

	// Configure IDF=4, ODF=2, NDIV=64 - this is 64Mhz
	CGM.FMPLL_CR.R = 0x12400000;
	
	// Switch to PLL
	ME.MCTL.R = 0x40005AF0;         /* Mode & Key */
	ME.MCTL.R = 0x4000A50F;         /* Mode & Key inverted */
	while(ME.GS.B.S_MTRANS==1) {};  /* Wait for mode entry to complete */
}

/**
 * Init the ADC reading port PB4 - connect to POT on the EVB (lower J18 pin)
 */
void initADC(void) 
{
    SIU.PCR[20].R = 0x2000;		// PB4 in analogue mode

    ADC_0.MCR.B.PWDN = 0;		/* ADC enable */
    ADC_0.MCR.B.MODE = 0; 		/* one-shot mode */
    ADC_0.MCR.B.CTUEN = 0;		/* start by SW */
    ADC_0.NCMR0.R = 1;			/* mask enable for channel0 */
    ADC_0.CIMR0.R = 1;			/* mask enable for channel0 */
    ADC_0.IMR.R = 2;			/* trigger */
}

void setupGlobalFrame() 
{
	int i;
	frame = (void *) txBuf;

	// Setup broadcast as destination
	for (i = 0; i < 6; i++) 
	{
		frame->destination[i] = broadcast[i];
	}

	// Setup source address
	for (i = 0; i < 6; i++) 
	{
		frame->source[i] = mac[i];
	}

	// Set length (static at 4 for the 32-bit data payload)
	frame->length = 4;
	
	// Setup txDesc for the (2x6-byte address + 2byte length + 4byte data) 18-byte frame
	txDesc[0].length = 18;
	txDesc[1].length = 18;
}

void rxEthernetFrame() 
{
	// Find the active recieve buffer
	BufferDescriptor *rxBufDesc = getActiveRxDesc();
	
	// Extract the frame
	MyEthernetFrame *rxFrame = (void *) rxBufDesc->bufferPointer;

	// Do something with it, if we want to - we check if it's from an FSL OUI
	if (rxFrame->source[0] == 0x0 && 
			rxFrame->source[1] == 0x4 && rxFrame->source[2] == 0x9F) 
	{
		EMIOS_0.CH[0].CADR.R = rxFrame->data; 	
	}

	// Mark the Rx buffer as available
	markRxDescEmpty(rxBufDesc);

	// Flag descriptors available to allow reception
	FEC.RDAR.R = 0x0100000;

	// Clear interrupt	
	FEC.EIR.R = 0x02000000;
}

void txEthernetFrame() 
{
	// Grab an ADC sample and put in our frame
	ADC_0.MCR.B.NSTART = 1;		/* ADC Go! */
	while (ADC_0.MSR.B.NSTART);	// Wait for scan to complete
	
	frame->data = ADC_0.CDR[0].B.CDATA;
	
	sendBuffer();
}

int main(void) {
	int i;
	
	STM.CR.R = 1;

	disableWatchdog();
	
	initModes();

	// Setup interrupts (rx frame handling)
	INTC_InitINTCInterrupts();
	INTC.CPR_PRC0.B.PRI = 0;
	INTC_InstallINTCInterruptHandler(rxEthernetFrame, 245, 10);
	
	setMacUsingStm();
	
	initADC();
	
	initFec();
	
	// Setup our frame that we'll send repeatedly
	setupGlobalFrame();

	// Setup eMIOS - this changes the duty cycle of PA0
	initEMIOS();
	initEMIOSch23();
	initEMIOSch0();
	
	// Wait for connection to be confirmed by the PHY
	while(!isConnected());

	while (1) 
	{
		txEthernetFrame();
		for (i = 0; i < 0xFFFF; i++);
	}
}



